/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package actor.components;

import actor.ActorComponent;
import logging.Logger;
import gameapp.GameApp;
import org.lwjgl.util.vector.Vector2f;
import org.w3c.dom.Node;
import util.MathUtil;
import util.XMLUtil;
import view.scene.PerspectiveCamera;

/**
 * Definert die Eigenschaften einer Kamera.
 * @author nico3000
 */
public class CameraComponent extends ActorComponent {
    /**
     * ID der Komponente.
     */
    public static final int ID = 0xd977f1e1;
    
    /**
     * Die Kamera der Komponente.
     */
    private final PerspectiveCamera camera = new PerspectiveCamera();

    @Override
    protected int getComponentId() {
        return ID;
    }

    @Override
    protected boolean init(Node data) {
        Vector2f z = new Vector2f(1e-1f, 1e+2f);
        float fov = 90.0f;
        float aspect = (float)GameApp.getGraphicsLayer().getWidth() / (float)GameApp.getGraphicsLayer().getHeight();
        while(data != null) {
            switch(data.getNodeName()) {
                case "Perspective":
                    fov = XMLUtil.getAttributeAsFloat(data, "fov", fov);
                    aspect = XMLUtil.getAttributeAsFloat(data, "aspect", aspect);
                    XMLUtil.getAttributesAsVector(data, "znear", "zfar", z);
                    break;
                case "#comment":
                case "#text":
                    // nothing to do
                    break;
                default:
                    Logger.INSTANCE.warning("node " + data.getNodeName() + " was ignored");
                    break;
            }
            data = data.getNextSibling();
        }
        return this.camera.init(this.getOwner(), MathUtil.toRadians(fov), aspect, z.x, z.y);
    }
    
    /**
     * Liefert die perspekivische Kamera dieser Komponente zurueck.
     * @return Die Kamera
     */
    public PerspectiveCamera getPerspectiveCamera() {
        return this.camera;
    }

    @Override
    protected void destroy() {
        super.destroy();
        this.camera.destroy();
    }
    
}
