/*
 * This software is the proprietary information of nico3000.
 */
package gameapp.audio;

import gameapp.GameApp;
import java.nio.FloatBuffer;
import logging.Logger;
import org.lwjgl.BufferUtils;
import org.lwjgl.openal.AL10;
import org.lwjgl.util.WaveData;
import org.lwjgl.util.vector.Vector3f;
import resource.ResHandle;
import resource.Resource;

/**
 *
 * @author nico3000
 */
public class AudioBuffer {
    private final String resource;
    private int buffer = 0;
    private int source = 0;
    private final Vector3f position = new Vector3f();

    public AudioBuffer(String resource) {
        this.resource = resource;
    }
    
    public boolean init() {
        if(this.buffer != 0 || this.source != 0) {
            Logger.INSTANCE.warning("AudioBuffer for %s is already initialized.");
            return false;
        }
        this.buffer = AL10.alGenBuffers();
        if(!GameApp.getSoundLayer().checkError()) {
            this.destroy();
            return false;
        }
        this.source = AL10.alGenSources();
        if(!GameApp.getSoundLayer().checkError()) {
            this.destroy();
            return false;
        }
        ResHandle handle = GameApp.getResCache().getHandle(new Resource(this.resource));
        if(handle == null) {
            Logger.INSTANCE.warning("Resource %s colud not be loaded.", this.resource);
            this.destroy();
            return false;
        }
        WaveData data = WaveData.create(handle.getBuffer());
        handle.unlock();
        if(data == null) {
            Logger.INSTANCE.warning("Failed to create wave data from resource %s.", this.resource);
            this.destroy();
            return false;
        }
        AL10.alBufferData(this.buffer, data.format, data.data, data.samplerate);
        if(!GameApp.getSoundLayer().checkError()) {
            this.destroy();
            return false;
        }
        AL10.alSourceQueueBuffers(this.source, this.buffer);
        if(!GameApp.getSoundLayer().checkError()) {
            this.destroy();
            return false;
        }
        this.updatePosition();
        return true;
    }
    
    public void destroy() {
        if(this.buffer != 0) {
            AL10.alDeleteBuffers(this.buffer);
            this.buffer = 0;
        }
        if(this.source != 0) {
            AL10.alDeleteSources(this.source);
            this.source = 0;
        }
    }
    
    public void setLooping(boolean looping) {
        AL10.alSourcei(this.source, AL10.AL_LOOPING, looping ? AL10.AL_TRUE : AL10.AL_FALSE);
    }
    
    public void setPositionRelative(boolean relative) {
        AL10.alSourcei(this.source, AL10.AL_SOURCE_RELATIVE, relative ? AL10.AL_TRUE : AL10.AL_FALSE);
    }
    
    public void setPosition(float x, float y, float z) {
        this.position.set(x, y, z);
        this.updatePosition();
    }
    
    private void updatePosition() {
        if(this.source != 0) {
            GameApp.getSoundLayer().checkError();
            AL10.alSource3f(this.source, AL10.AL_POSITION, this.position.x, this.position.y, this.position.z);
            GameApp.getSoundLayer().checkError();
        }
    }
    
    public void play() {
        if(this.source != 0) {
            AL10.alSourcePlay(this.source);
        }
    }
    
    public void pause() {
        if(this.source != 0) {
            AL10.alSourcePause(this.source);
        }
    }
    
    public void stop() {
        if(this.source != 0) {
            AL10.alSourceStop(this.source);
        }
    }

    public void rewind() {
        if(this.source != 0) {
            AL10.alSourceRewind(this.source);
        }
    }    
}
