/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package graphics.shader;

import java.nio.ByteBuffer;
import logging.Logger;
import gameapp.GameApp;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL20;
import resource.Resource;

/**
 * Oberklasse fuer einen Shader.
 * @author nico3000
 */
public abstract class AbstractShader {
    /**
     * Die ID des Shaderobjekts (Rueckgabe von <tt>glCreateShader()</tt>).
     */
    private int id = 0;
    
    /**
     * Initialisierung mittels einer Resource, die den Sorucecode des Shaders
     * enthaelt.
     * @param resource Die Resource mit dem Sourcecode
     * @return <tt>true</tt>, falls erfolgreich
     */
    public boolean init(String resource) {
        ByteBuffer buffer = GameApp.getGraphicsLayer().resource2ByteBuffer(new Resource(resource));
        if(buffer == null) {
            return false;
        }
        this.id = GL20.glCreateShader(this.getType());
        if(this.id >= 0) {
            GL20.glShaderSource(this.id, buffer);
            GL20.glCompileShader(this.id);
            if(GL20.glGetShaderi(this.id, GL20.GL_COMPILE_STATUS) == GL11.GL_FALSE) {
                String log = GL20.glGetShaderInfoLog(this.id, 1024);
                Logger.INSTANCE.error(log);
            } else {
                return true;
            }
        }
        this.destroy();
        return false;
    }
    
    /**
     * Loescht den Shader.
     */
    public void destroy() {
        if(this.id != 0) {
            GL20.glDeleteShader(this.id);
            this.id = 0;
        }
    }   

    /**
     * Getter fuer die ID des Shaders.
     * @return Die ID
     */
    public int getId() {
        return this.id;
    } 
    
    /**
     * Liefert den Typ des Shaders (VS, GS, FS, ...).
     * @return Typ des Shaders
     */
    protected abstract int getType();
}
