/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package graphics.shader;

import java.nio.FloatBuffer;
import logging.Logger;
import org.lwjgl.opengl.GL15;
import org.lwjgl.opengl.GL30;
import org.lwjgl.opengl.GL31;

/**
 * In einem UniformBuffer liegen die Daten eines GLSL UniformBlock.
 * @author nico3000
 */
public class UniformBuffer { 
    /**
     * GLSL: CameraBlock.
     */
    public static final int BINDING_CAMERA = 1;
    /**
     * GLSL: ModelBlock.
     */
    public static final int BINDING_MODEL = 2;
    /**
     * GLSL: MaterialBlock.
     */
    public static final int BINDING_MATERIAL = 3;
    
    /**
     * Die ID des Buffers.
     */
    private int id = 0;
    private int baseIndex = -1;
    
    /**
     * Initialisiert den Buffer.
     * @return <tt>true</tt>, falls erfolgreich
     */
    public boolean init() {
        if(this.id != 0) {
            Logger.INSTANCE.warning("Initializing already initialized UniformBuffer.");
            this.destroy();
        }
        this.id = GL15.glGenBuffers();
        return this.id != 0;
    }
    
    /**
     * Gibt den Buffer wieder frei.
     */
    public void destroy() {
        if(this.id != 0) {
            GL15.glDeleteBuffers(this.id);
            this.id = 0;
        }
    }
    
    /**
     * Wurde zuvor der BindingIndex mit <tt>setBindingIndex</tt> gesetzt, so
     * bindet diese Methode den Buffer an den gesetzten Index.
     */
    public void bind() {
        if(this.baseIndex != -1) {
            GL30.glBindBufferBase(GL31.GL_UNIFORM_BUFFER, this.baseIndex, this.id);
        }
    }
    
    /**
     * Setzt die Daten des Buffers.
     * @param data Die Daten
     */
    public void setData(FloatBuffer data) {
        GL15.glBindBuffer(GL31.GL_UNIFORM_BUFFER, this.id);
        GL15.glBufferData(GL31.GL_UNIFORM_BUFFER, data, GL15.GL_DYNAMIC_DRAW);
//        String out = data.capacity() + ": ";
//        for(int i=0; i < data.capacity(); ++i) {
//            out += String.format("%.3f ", data.get(i));
//        }
//        Logger.INSTANCE.info(out);
    }
    
    /**
     * Setzt den Index fest, den zukuenftige Aufrufe von <tt>bind()</tt>
     * benutzen.
     * @param index Der Index
     */
    public void setBindingIndex(int index) {
        this.baseIndex = index;
    }
}
