/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package input;

/**
 * Interface fuer eine Klasse, die auf Mausbewegungen reagieren kann.
 * @author nico3000
 */
public interface PointerHandler {
    /**
     * Wird aufgerufen, wenn die Maus sich bewegt hat.
     * @param posX Die aktuelle x-Position
     * @param posY Die aktuelle y-Position
     * @param dX Die Bewegung der Maus in x-Richtung
     * @param dY Die Bewegung der Maus in y-Richtung
     * @return <tt>true</tt>, falls die Bewegung verarbeitet wurde; false, wenn sie
     *         ignoriert wurde und der naechste Handler benachrichtigt werden
     *         soll
     */
    public boolean pointerMoved(int posX, int posY, int dX, int dY);
    
    /**
     * Wird aufgerufen, wenn ein Button an der Maus gedrueckt wurde.
     * @param button Der Button
     * @return <tt>true</tt>, falls die Bewegung verarbeitet wurde; false, wenn sie
     *         ignoriert wurde und der naechste Handler benachrichtigt werden
     *         soll
     */
    public boolean onButtonDown(int button);
    
    /**
     * Wird aufgerufen, wenn ein Button an der Maus losgelassen wurde.
     * @param button Der Button
     * @return <tt>true</tt>, falls die Bewegung verarbeitet wurde; false, wenn sie
     *         ignoriert wurde und der naechste Handler benachrichtigt werden
     *         soll
     */
    public boolean onButtonUp(int button);
    
    /**
     * Wird aufgerufen, wenn das Rad der Maus gedreht wurde.
     * @param delta Die Anzahl Einheiten der Drehung
     * @return <tt>true</tt>, falls die Bewegung verarbeitet wurde; false, wenn sie
     *         ignoriert wurde und der naechste Handler benachrichtigt werden
     *         soll
     */
    public boolean onWheelMoved(int delta);
}
