/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package mesh;

import logging.Logger;
import mesh.loader.GeometryLoader;
import mesh.loader.wavefront.WavefrontObjLoader;
import org.w3c.dom.Node;

/**
 * Der Meshloader ist die zentrale Anlaufstelle, um Meshes zu erzeugen.
 * @author nico3000
 */
public class MeshLoader {
    /**
     * Ein Platzhaltermesh, dass anstelle von noch nicht geladenen Meshes
     * angezeigt wird.
     */
    public static Mesh DEFAULT_MESH;
    
    /**
     * Initialisiert das <tt>DEFAULT_MESH</tt>.
     * @return <tt>true</tt>, falls erfolgreich
     */
    public static boolean init() {
        DEFAULT_MESH = MeshLoader.loadMesh("Meshes/exclamation.obj", null);
        if(DEFAULT_MESH == null) {
            Logger.INSTANCE.error("Default mesh could not be initialized!");
            return false;
        } else {
            return true;
        }
    }
    
    /**
     * Gibt das <tt>DEFAULT_MESH</tt> wieder frei.
     */
    public static void destroy() {
        DEFAULT_MESH.release();
    }
    
    /**
     * Laedt ein Mesh aus einer Resource und initialisiert es.
     * @param resource Die Resource
     * @param additionalData Der Knoten, der das Mesh referenziert hat
     * @return Das Mesh, falls erfolgreich initialisiert; sonst <tt>null</tt>
     */
    public static Mesh loadMesh(String resource, Node additionalData) {
        int endingStart = resource.lastIndexOf('.');
        if(endingStart == -1) {
            Logger.INSTANCE.error("No resource file ending (%s) for mesh!", resource);
            DEFAULT_MESH.addRef();
            return DEFAULT_MESH;
        }
        String ending = resource.substring(endingStart);
        GeometryLoader loader;
        switch(ending) {
            case ".obj":
                loader = new WavefrontObjLoader(resource);
                break;
            default:
                Logger.INSTANCE.error("Resource file ending %s not known!", ending);
                DEFAULT_MESH.addRef();
                return DEFAULT_MESH;
        }
        loader.setAdditionalData(additionalData);
        Mesh mesh = Mesh.createMesh(loader);
        if(mesh == null) {
            DEFAULT_MESH.addRef();
            return DEFAULT_MESH;
        } else {
            return mesh;
        }
    }
}
