/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package resource.loader;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import javax.imageio.ImageIO;
import logging.Logger;
import resource.ResHandle;

/**
 * Wandelt die Daten einer Bilddatei in eine Floatfolge um.
 * @author nico3000
 */
public class ImageLoader implements ResourceLoader {
    @Override
    public String getPattern() {
        return "(.*)((.jpg)|(.png)|(.bmp)|(.cur))";
    }

    @Override
    public boolean useRawFile() {
        return false;
    }

    @Override
    public int getLoadedResourceSize(byte[] buffer, int rawSize) {
        BufferedImage image;
        try {
            image = ImageIO.read(new ByteArrayInputStream(buffer));
        } catch (IOException ex) {
            Logger.INSTANCE.error(ex.getMessage());
            return 0;
        }
        if(image == null) {
            return 0;
        }
        int width = image.getWidth();
        int height = image.getHeight();
        int channels = image.getColorModel().getNumComponents();
        return 4 * width * height * channels;
    }

    @Override
    public boolean loadResource(byte[] buffer, int rawSize, ResHandle handle) {
        BufferedImage image;
        try {
            image = ImageIO.read(new ByteArrayInputStream(buffer));
        } catch (IOException ex) {
            Logger.INSTANCE.error(ex.getMessage());
            return false;
        }
        if(image == null) {
            return false;
        }
        ByteBuffer byteBuffer = ByteBuffer.wrap(handle.getWritableBuffer());
        for(int y=0; y < image.getHeight(); ++y) {
            for(int x=0; x < image.getWidth(); ++x) {
                for(int c=0; c < image.getColorModel().getNumComponents(); ++c) {
                    byteBuffer.putFloat(image.getRaster().getSampleFloat(x, y, c) / 255.0f);
                }
            }
        }
        ImageExtraData extra = new ImageExtraData(image.getWidth(), image.getHeight(), image.getColorModel().getNumComponents());
        handle.setExtra(extra);
        return true;
    }
    
}
