/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package util;

import graphics.geometry.IndexBuffer;
import graphics.geometry.VertexBuffer;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import mesh.Mesh;
import mesh.loader.generic.GenericLoader;
import org.lwjgl.BufferUtils;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;

/**
 * Erzeugt haeufig benutzte Geometrien.
 * @author nico3000
 */
public class GenericMeshFactory {	
    /**
     * Erzeugt einen Quader.
     * @param sizeX Kantenlaenge in x-Richtung
     * @param sizeY Kantenlaenge in y-Richtung
     * @param sizeZ Kantenlaenge in z-Richtung
     * @param posX Koordinate des Mittelpunkts
     * @param posY Koordinate des Mittelpunkts
     * @param posZ Koordinate des Mittelpunkts
     * @return Der Quader
     */
    public static Mesh createCuboid(float sizeX, float sizeY, float sizeZ, float posX, float posY, float posZ) {
        Vector3f positions[] = {
            new Vector3f(posX - 0.5f * sizeX, posY - 0.5f * sizeY, posZ - 0.5f * sizeZ),
            new Vector3f(posX + 0.5f * sizeX, posY - 0.5f * sizeY, posZ - 0.5f * sizeZ),
            new Vector3f(posX + 0.5f * sizeX, posY + 0.5f * sizeY, posZ - 0.5f * sizeZ),
            new Vector3f(posX - 0.5f * sizeX, posY + 0.5f * sizeY, posZ - 0.5f * sizeZ),
            new Vector3f(posX - 0.5f * sizeX, posY - 0.5f * sizeY, posZ + 0.5f * sizeZ),
            new Vector3f(posX + 0.5f * sizeX, posY - 0.5f * sizeY, posZ + 0.5f * sizeZ),
            new Vector3f(posX + 0.5f * sizeX, posY + 0.5f * sizeY, posZ + 0.5f * sizeZ),
            new Vector3f(posX - 0.5f * sizeX, posY + 0.5f * sizeY, posZ + 0.5f * sizeZ),
        };
        FloatBuffer vertices = BufferUtils.createFloatBuffer(8 * 4 * 6);
        IntBuffer indices = BufferUtils.createIntBuffer(5 * 6);
        // 0123 -z
        Vector3f normal = new Vector3f(0.0f, 0.0f, -1.0f);
        positions[0].store(vertices); normal.store(vertices); new Vector2f(0.0f, 0.0f).store(vertices);
        positions[1].store(vertices); normal.store(vertices); new Vector2f(1.0f, 0.0f).store(vertices);
        positions[3].store(vertices); normal.store(vertices); new Vector2f(0.0f, 1.0f).store(vertices);
        positions[2].store(vertices); normal.store(vertices); new Vector2f(1.0f, 1.0f).store(vertices);
        indices.put(new int[] { 0, 1, 2, 3, IndexBuffer.RESTART });
        // 4037 -x
        normal = new Vector3f(-1.0f, 0.0f, 0.0f);
        positions[4].store(vertices); normal.store(vertices); new Vector2f(0.0f, 0.0f).store(vertices);
        positions[0].store(vertices); normal.store(vertices); new Vector2f(1.0f, 0.0f).store(vertices);
        positions[7].store(vertices); normal.store(vertices); new Vector2f(0.0f, 1.0f).store(vertices);
        positions[3].store(vertices); normal.store(vertices); new Vector2f(1.0f, 1.0f).store(vertices);
        indices.put(new int[] { 4, 5, 6, 7, IndexBuffer.RESTART });
        // 5476 +z
        normal = new Vector3f(0.0f, 0.0f, 1.0f);
        positions[5].store(vertices); normal.store(vertices); new Vector2f(0.0f, 0.0f).store(vertices);
        positions[4].store(vertices); normal.store(vertices); new Vector2f(1.0f, 0.0f).store(vertices);
        positions[6].store(vertices); normal.store(vertices); new Vector2f(0.0f, 1.0f).store(vertices);
        positions[7].store(vertices); normal.store(vertices); new Vector2f(1.0f, 1.0f).store(vertices);
        indices.put(new int[] { 8, 9, 10, 11, IndexBuffer.RESTART });
        // 1562 +x
        normal = new Vector3f(1.0f, 0.0f, 0.0f);
        positions[1].store(vertices); normal.store(vertices); new Vector2f(0.0f, 0.0f).store(vertices);
        positions[5].store(vertices); normal.store(vertices); new Vector2f(1.0f, 0.0f).store(vertices);
        positions[2].store(vertices); normal.store(vertices); new Vector2f(0.0f, 1.0f).store(vertices);
        positions[6].store(vertices); normal.store(vertices); new Vector2f(1.0f, 1.0f).store(vertices);
        indices.put(new int[] { 12, 13, 14, 15, IndexBuffer.RESTART });
        // 4510 -y
        normal = new Vector3f(0.0f, -1.0f, 0.0f);
        positions[4].store(vertices); normal.store(vertices); new Vector2f(0.0f, 0.0f).store(vertices);
        positions[5].store(vertices); normal.store(vertices); new Vector2f(1.0f, 0.0f).store(vertices);
        positions[0].store(vertices); normal.store(vertices); new Vector2f(0.0f, 1.0f).store(vertices);
        positions[1].store(vertices); normal.store(vertices); new Vector2f(1.0f, 1.0f).store(vertices);
        indices.put(new int[] { 16, 17, 18, 19, IndexBuffer.RESTART });
        // 3267 +y
        normal = new Vector3f(0.0f, 1.0f, 0.0f);
        positions[3].store(vertices); normal.store(vertices); new Vector2f(0.0f, 0.0f).store(vertices);
        positions[2].store(vertices); normal.store(vertices); new Vector2f(1.0f, 0.0f).store(vertices);
        positions[7].store(vertices); normal.store(vertices); new Vector2f(0.0f, 1.0f).store(vertices);
        positions[6].store(vertices); normal.store(vertices); new Vector2f(1.0f, 1.0f).store(vertices);
        indices.put(new int[] { 20, 21, 22, 23, IndexBuffer.RESTART });

        vertices.position(0);
        indices.position(0);
        return createMesh(indices, IndexBuffer.TRIANGLE_STRIP, vertices);
    }
    
    public static Mesh createSphere(int subDivide, float radius, float posX, float posY, float posZ) {
        FloatBuffer vertices = BufferUtils.createFloatBuffer(8 * (subDivide + 1) * (subDivide / 2 + 1));
        IntBuffer indices = BufferUtils.createIntBuffer((2 * subDivide + 2 + 1) * (subDivide / 2));
        float theta = 0.0f;
        float dPhi = MathUtil.PI_MUL2 / (float)subDivide;
        float dTheta = MathUtil.PI / (float)(subDivide / 2);
        for(int iTheta=0; iTheta <= subDivide / 2; ++iTheta, theta += dTheta) {
            float phi = 0.0f;
            for(int iPhi=0; iPhi <= subDivide; ++iPhi, phi += dPhi) {
                float sinTheta = MathUtil.sin(theta);
                float cosTheta = MathUtil.cos(theta);
                float sinPhi = MathUtil.sin(phi);
                float cosPhi = MathUtil.cos(phi);
                Vector3f normal = new Vector3f(sinTheta * cosPhi, sinTheta * sinPhi, cosTheta);
                new Vector3f(posX + radius * normal.x, posY + radius * normal.y, posZ + radius * normal.z).store(vertices);
                normal.store(vertices);
                new Vector2f(phi / MathUtil.PI_MUL2, theta / MathUtil.PI).store(vertices);
                
                if(iTheta != subDivide / 2) {
                    indices.put((iTheta + 1) * (subDivide + 1) + iPhi);
                    indices.put(iTheta * (subDivide + 1) + iPhi);
                }
            }
            if(iTheta != subDivide / 2) {
                indices.put(IndexBuffer.RESTART);
            }
        }
        indices.position(0);
        vertices.position(0);
        return createMesh(indices, IndexBuffer.TRIANGLE_STRIP, vertices);
    }
    
    /**
     * Erzeugt eine fertig initialisierte Geometrie aus Index- und
     * Vertexbuffern. Die Vertices muessen dabei folgendermassen aufgebaut sein.
     * <ul>
     *   <li>Position, 3 Floats</li>
     *   <li>Normal, 3 Floats</li>
     *   <li>Texturkoordinaten, 2 Floats</li>
     * </ul>
     * @param indices Der Indexbuffer
     * @param topology Die Topologie des Indexbuffers
     * @param vertices Der Vertexbuffer
     * @return Die Geometrie oder <tt>null</tt> bei Fehlern
     */
    private static Mesh createMesh(IntBuffer indices, int topology, FloatBuffer vertices) {
        VertexBuffer vertexBuffer = new VertexBuffer();
        IndexBuffer indexBuffer = new IndexBuffer();
        if(vertexBuffer.init() && indexBuffer.init()) {
            indexBuffer.setData(indices, topology);
            vertexBuffer.setData(vertices);
            vertexBuffer.addAttribute(VertexBuffer.ATTR_POS, 3);
            vertexBuffer.addAttribute(VertexBuffer.ATTR_NORMAL, 3);
            vertexBuffer.addAttribute(VertexBuffer.ATTR_TEX, 2);
            GenericLoader loader = new GenericLoader(vertexBuffer, indexBuffer);
            Mesh mesh = Mesh.createMesh(loader);
            if(mesh == null) {
                vertexBuffer.destroy();
                indexBuffer.destroy();
                return null;
            } else {
                return mesh;
            }
        }
        return null;
    }
}
