package view.scene.nodes;

import logging.Logger;
import util.Pose;
import view.scene.SceneScreenElement;
import actor.Actor;
import actor.components.TransformComponent;
import event.EventData;
import event.EventListener;
import event.events.ActorMovedEvent;
import gameapp.GameApp;

public class InvisibleNode extends SceneNode implements EventListener {

	
    /**
     * Die Pose des Actors.
     */
    private Pose pose;
		
    public boolean init(int actorId) {
        Actor actor = GameApp.getGameLogic().getActor(actorId);
        if(actor == null) {
            Logger.INSTANCE.error("invalid actor id");
            return false;
        }
        TransformComponent transformComponent = (TransformComponent)actor.getComponent(TransformComponent.ID);
        if(transformComponent == null) {
            Logger.INSTANCE.error("actor has no transform component");
            return false;
        }
        this.pose = transformComponent.getPose();
        GameApp.getEventManager().register(this, ActorMovedEvent.ID);
        super.setActorId(actorId);
        return true;
    }

    
	@Override
	public boolean isVisible(SceneScreenElement scene) {
		return true;
	}

	@Override
	public void preRender(SceneScreenElement scene) {
		  scene.getStack().push(this.pose.getModel(), this.pose.getModelInv());
	}

	@Override
	public void render(SceneScreenElement scene) {
	}

	@Override
	public void postRender(SceneScreenElement scene) {
		scene.getStack().pop();
	}


	@Override
	public void trigger(EventData data) {
        switch(data.getId()) {
        case ActorMovedEvent.ID:
            if(this.getActorId() == ((ActorMovedEvent)data).getActorId()) {
                this.actorMoved();
            } else {
            }
            break;
    }		
	}

}
